/***************************************************************************
                           GanttTask.java  -  description
                             -------------------
    begin                : dec 2002
    copyright            : (C) 2002 by Thomas Alexandre
    email                : alexthomas@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

package net.sourceforge.ganttproject;

import java.awt.Color;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;

import net.sourceforge.ganttproject.resource.HumanResource;
import net.sourceforge.ganttproject.shape.ShapeConstants;
import net.sourceforge.ganttproject.shape.ShapePaint;
import net.sourceforge.ganttproject.task.TaskManager;

/**
 * Class that generate a task
 */

public class GanttTask

    implements Serializable {

  public static int LOW = 0;

  public static int NORMAL = 1;

  public static int HIGHT = 2;

  /** Notes of the task */

  private String notes;

  /** Begining date of the task */

  private GanttCalendar start;

  /* Duration of the task. */

  private int length;

  /** Name of the task. */

  private String name;

  /** Depend list of the task */

  //private ArrayList depend = new ArrayList();

  
  /** Id of the task. */
  
  private String id="";
  
  /** State between 0% and 100%*/

  private int percent;

  /**Is it a meeting point( false by default) */

  private boolean bilan;

  /** The list of the users assigned to this task */

  private ArrayList users = new ArrayList();

  /** Color to print task */

  private Color color;

  /** The priority of the task */

  private int priority = 1;
  
  /** the shape used to print this task*/
   private ShapePaint myShape ;

/////////////////////////////////////////////////////////////////////////////////

  /** Constructor */

  public GanttTask(String name, GanttCalendar start, int length) {

    this.name = name;

    this.start = start;

    this.length = length;

    this.percent = 0;

    this.bilan = false;

    this.notes = new String();

  }

  /** Return a clone of this task */
  /*
    public GanttTask Clone() {
      GanttTask res = new GanttTask(name, start, length);
      res.bilan = bilan;
      for (int i = 0; i < depend.size(); i++) {
        res.depend.add(depend.get(i));
      }
      for (int i = 0; i < users.size(); i++) {
        res.users.add(users.get(i));
      }
      res.notes = notes;
      res.percent = percent;
      res.priority = priority;
      res.color = (color == null) ? null :
          new Color(color.getRed(), color.getGreen(), color.getBlue());
      return res;
    }*/

  /**
   * clone the GanttTask
   * @return
   */
  public GanttTask Clone() {
    GanttTask res = new GanttTask(name, start, length);
    res.bilan = bilan;

    for (int i = 0; i < predecessors.size(); i++) {
      GanttTaskRelationship tempRel = (GanttTaskRelationship) ( (
          GanttTaskRelationship) predecessors.get(i)).clone();
      addPredecessor(tempRel);
    }

    for (int i = 0; i < successors.size(); i++) {
      GanttTaskRelationship tempRel = (GanttTaskRelationship) ( (
          GanttTaskRelationship) successors.get(i)).clone();
      addSuccessor(tempRel);
    }

    for (int i = 0; i < users.size(); i++) {
      res.users.add(users.get(i));
    }
    res.notes = notes;
	res.id = new String (id);
    res.percent = percent;
    res.priority = priority;
    res.color = (color == null) ? null :
        new Color(color.getRed(), color.getGreen(), color.getBlue());
    return res;
  }

  /** Return the name. */
  public String toString() {
    //return id+" "+name;
    return name;
  }
  
  public String getName() {
  	return name;
  }


  /** return the identity of the task */
  public String getID () {
    return id;
  }

  /** Change the identity of the task */
  public void setID (String newID){
    id = newID;
  }
  
  /** Indent the id of the task */
  public void indentID (String parent){
    id= parent+"1.";
  }
  
  /** Return the next id of the task */
  public String nextID(){
    if(id.length()<2) return id;
  	String last_id = id.substring(id.length()-2, id.length()-1);
	int id_int=new Integer(last_id).intValue();
	id_int++;
	return id.substring(0, id.length()-2) + ""+id_int+".";
  }

  /** Return the previous id of the task */
  public String prevID(){
    if(id.length()<2) return id;
  	String last_id = id.substring(id.length()-2, id.length()-1);
	int id_int=new Integer(last_id).intValue();
	if(id_int>0)id_int--;
	return id.substring(0, id.length()-2) + ""+id_int+".";
  }
  
  /** Return the shape of the task */
  public ShapePaint getShape() {
     if (this.myShape == null) {
      ShapePaint pattern =ShapeConstants.BACKSLASH;
      return new ShapePaint(pattern, getColor() , getColor());
      }
     return myShape;

    }

  /** Set a new shape for the task */
  public void setShape(ShapePaint newShape)
  {
     this.myShape = newShape;

    }

  /** Return the duration */

  public int getLength() {

    return length;

  }
  
  /**
   * Allows to determine, if a special shape is defined for this task.
   * @return true, if this task has its own shape defined.
   */
  public boolean shapeDefined()
  {
    return (myShape != null);
  }

  /** Return the begining */

  public GanttCalendar getStart() {

    return start;

  }

  /** Return the end date */

  public GanttCalendar getEnd() {

    GanttCalendar end = start.Clone();

    end.add(length);

    return end;

  }

  /** Return the notes */

  public String getNotes() {

    return notes;

  }

  /** Return the state */

  public int getPercent() {

    return percent;

  }

  /** Return the list of depends */
  //replaced by getSuccessors()
  /*
     public ArrayList getDepend() {
    return depend;
     }*/

  /** Is it a meeting point?? */
  public boolean getBilan() {
    return bilan;
  }

  /** Return the priority of the task */
  public int getPriority() {
    return priority;
  }

  /** Change the name */
  public void setName(String name) {
    this.name = name;
  }

  /** Change the duration */
  public void setLength(int l) {
    length = l;
  }

  /** Change the begining */
  public void setStart(GanttCalendar s) {
    start = s.Clone();
  }

  /** Change the end of the date and the length indead */

  public void setEnd(GanttCalendar e) {

    //By CL

    GanttCalendar end = e.Clone();

    length = start.diff(end);

    //End By CL

  }

  /*
     // joshh -- added setEnd function April 20, 2003
   public void setEnd(GanttCalendar e) {
    GanttCalendar end = start.Clone();
    end.add(length);
    // Is new end date 'e'  before end date 'end'
    if (e.compareTo(end) < 0) {
      int len = e.diff(end); // Number of overlapping days
      setStart(start.newAdd( -len));
    }
     }*/

  /** Change notes */

  public void setNotes(String note) {

    notes = note;

  }
  
  /** Add  notes */

  public void addNotes(String note) {

    notes += note;

  }

  /** Change state */

  public void setPercent(int percent) {

    this.percent = percent;

  }

  /** Change meeting point */

  public void setBilan(boolean b) {

    bilan = b;

  }

  /** Add new task on depends list */
  //replaced by addSuccessor(GanttTaskRelationshp successor);
  /*
     public void setDepend(String t) {
    //to avoid repete-By CL
    for(int i=0; i<depend.size(); i++){
      if(((String)depend.get(i)).equals(t)){
        return;
      }
    }
    //--END By CL
    depend.add(t);
     }*/

  /** Set the priority for the task */

  public void setPriority(int p) {

    priority = p;

  }

  /** Clear depends list */
  /*
     public void clearDepend() {
    depend.clear();
     }*/

  /** This function look on the array if the task named "t" is in.
   * In this case, replace by a new name "n". */
  /*
     public void checkDepend(String t, String n) {
    for (int i = 0; i < depend.size(); i++) {
      if (depend.get(i).toString() == t) {
        depend.remove(i);
        depend.add(n);
      }
    }
     }*/
//!@#
  /**supposed method that replaces checkDepend(String t, String n)
   *
   * replace the old successor (oldRel) with new one (newRel)
   * @param oldRel old successor
   * @param newRel new relationship
   */
  public void replaceOldSuccessor(GanttTaskRelationship oldRel,
                                  GanttTaskRelationship newRel) {
    for (int i = 0; i < successors.size(); i++) {
      GanttTaskRelationship tempRel = (GanttTaskRelationship) successors.get(i);
      if (tempRel.equals(oldRel)) {
        oldRel = newRel;
      }
    }
  }

  /**
   * This function calculate the begin of a task in function of the sub taks.
   */

  public void refreshDateAndAdvancement(GanttTree tree) {

    if (tree.getNode(getTaskID()/*name*/).isRoot()) {

      return;

    }

    ArrayList l = tree.getAllChildTask(this/*name*/);

    if (l.size() == 0) {

      return;

    }

    GanttTask taskMin = (GanttTask) ( (DefaultMutableTreeNode) l.get(0)).

        getUserObject();

    GanttTask taskMax = (GanttTask) ( (DefaultMutableTreeNode) l.get(0)).

        getUserObject();

    int sumDay = taskMin.length;

    ;

    float sum = (float) taskMin.length * (float) taskMin.percent / 100;

    ;

    for (int i = 1; i < l.size(); i++) {

      GanttTask task = (GanttTask) ( (DefaultMutableTreeNode) l.get(i)).

          getUserObject();

      //Search for the min task

      if (taskMin.start.compareTo(task.start) == 1) {

        taskMin = task;

        //Search for the max task

      }

      if (taskMax.start.newAdd(taskMax.length).compareTo(task.start.newAdd(task.

          length)) == -1) {

        taskMax = task;

        //Calcul the state

      }

      if (!task.bilan) {

        sumDay += task.length;

        sum += (float) task.length * (float) task.percent / 100;

      }

    }

    //Change the state

    this.percent = (int) (sum * 100 / (float) sumDay);

    this.setStart(taskMin.start.Clone());

    this.setLength(taskMin.start.diff(taskMax.start.newAdd(taskMax.length)));

  }

  /** Return the list of the user assigned to */
  public ArrayList getUsersList() {
    return users;
  }

  /** This adds a user to a task.
   * The method creates a Hashtable and places this into an
   * ArrayList. This is for now and will be changed to
   * be more convenient in the future for other developers.
   * barmeier
   * @param user is the HumanResource object that represents a
   * user.
   * @param load is the load this task produes on the resource represented
   * through the user object.
   */
  public void taskUser(HumanResource user, int load) {
    if (load == 0) { // if the user has no load in this task. He is automatically unassigned
      removeUser(user);
      return;
    }
    if (!isIn(user)) {
      Hashtable store = new Hashtable();
      store.put("NAME", user.getName()); // this is for convenience
      store.put("LOAD", new Integer(load));
      store.put("USER_OBJECT", user);
      users.add(store);
    }
    else { // this is an update !
      Hashtable store = findUser(user);
      int taskLoad = ( (Integer) store.get("LOAD")).intValue();
// this is to simple :( I think I have to alculate this on the fly when needed
//            user.setLoad((user.getLoad()-taskLoad)+load);
    }
  }

  /** Remove the user u_name from the task's user list */
  public void removeUser(HumanResource user) {
    users.remove(findUser(user));
  }

  /** Looks for a given user object in this task. If the user is assigned
   * to this task the method returns the Hashtable where the additional
   * informations like load,... are stored.
   * @param user the user the method should search for
   * @return the hashtable where the users additional informations
   * concerning this task are stored. The method returns null
   * if the user could not be found.
   *
   * Be aware of Nullpointer Exceptions ;)
   */
  public Hashtable findUser(HumanResource user) {
    for (int i = 0; i < users.size(); i++) {
      Hashtable h = (Hashtable) users.get(i);
      if (h.get("USER_OBJECT").equals(user)) {
        return h;
      }
    }
    return null;
  }

  public void cleanUsers() {
    users.clear();
  }

  public ArrayList getUserNames() {
    ArrayList userNames = new ArrayList();
    for (int i = 0; i < users.size(); i++) {
      Hashtable h = (Hashtable) users.get(i);
      userNames.add(h.get("NAME"));
    }
    return userNames;
  }

  public boolean isIn(HumanResource user) {
    return findUser(user) != null;
  }

  /**
   * @return Color of this task
   */

  public Color getColor() {

    if (color == null) {

      return GanttGraphicArea.taskDefaultColor;

    }

    return color;

  }

  /**
   * @param color New color for this task or null for default color
   */

  public void setColor(Color color) {

    this.color = color;

  }

  /**
   * Allows to determine, if a special color is defined for this task.
   * @return true, if this task has its own color defined.
   */

  public boolean colorDefined() {

    return (color != null);

  }

  /////////////////////////////////////////////////////////////

  //Added 13-May-2003/ CL

  private Vector predecessors = new Vector(); //vector store all predecessors' relationships

  private Vector successors = new Vector(); //vector store all successors' relationship

  /**whether the time relationship between this task and other has been checked. property will be used in scheduling check*/

  private boolean checked = false;

  /**get all the successors*/

  public Vector getSuccessors() {

    return successors;

  }

  /**set all the successors*/

  public void setSuccessors(Vector successors) {
    this.successors.clear();
    for(int i=0; i<successors.size(); i++){
      addSuccessor((GanttTaskRelationship)successors.get(i));
    }
  }


  /**add one successor*/

  public void addSuccessor(GanttTaskRelationship successor) { //!@#
  //System.out.println("AddSuccessors : "+successor);
  
    if(!whetherHasRelationship(successors, successor)){ //if already has this relationships
      this.successors.add(successor);

      //if the successor does not has this task as predecessor, add it here
      GanttTask sucTask = GanttTask.getTaskByTaskID(successor.
          getSuccessorTaskID());
		  
		 
      if (sucTask != null) {
        
		GanttTaskRelationship predRel = new GanttTaskRelationship(this.taskID, -1,
            successor.getRelationshipType());

        if (!whetherHasRelationship(sucTask.getPredecessors(), predRel)) {
          sucTask.addPredecessor(predRel);
        }
      }
      else {
        //if program might be here at the time when reading from a XML file.
        //But i have add corresponding
        //code in GanttTaskXMLOpen.constructRelationship :)
      }
    }
  }

  /**
   * Check whether the relationships vector already has the rel
   * @param relationships vector of relationships
   * @param rel check relatinonship
   * @return
   */
  private boolean whetherHasRelationship(Vector relationships,
                                         GanttTaskRelationship rel) {
    GanttTaskRelationship taskRel;
    for (int i = 0; i < relationships.size(); i++) {
      taskRel = (GanttTaskRelationship) relationships.get(i);
      if (taskRel.equals(rel)) {
        return true;
      }
    }
    return false;
  }

  /**set all the predecessors*/

  public void setPredecessors(Vector predecessors) {
    this.predecessors.clear();
    for(int i=0; i<predecessors.size(); i++){
      addPredecessor((GanttTaskRelationship)predecessors.get(i));
    }
  }


  public void setResources(ArrayList resources) {
    users = resources; // FIXME: This should be generic for all resources if possible
  }

  /**add one predecessor*/

  public void addPredecessor(GanttTaskRelationship predecessor) {
  //System.out.println("addPredecessor : "+predecessor);
    if(!whetherHasRelationship(predecessors, predecessor)){//check if already has predecessor relationship
      this.predecessors.add(predecessor);
      GanttTask predTask = GanttTask.getTaskByTaskID(predecessor.
          getPredecessorTaskID());
      if (predTask != null) {
        if(predecessor.getRelationshipType()==0){
          //System.out.println("Catch me in GanttTask.12222wsd");
        }
        GanttTaskRelationship succRel = new GanttTaskRelationship( -1,
            this.taskID,
            predecessor.getRelationshipType());
        //if the predecessor task does not has this task as successor, add it here
        if (!whetherHasRelationship(predTask.getSuccessors(), succRel)) {
		   predTask.addSuccessor(succRel);
        }
      }else{
        System.out.println("You should not read me or turn to code in GanttTask.addPredecssor()");
      }
    }
    /**get all the predecessor*/

  }

  public Vector getPredecessors() {

    return predecessors;

  }
  
  /** Unlink the task from all relationship */
  public void unlink () {
  	/* This function remove the relationships connected to the current task
	   Of course clear the 2 array with successors and predecessors. But also 
	   Remove from the differents suc. and pred. the current task */
	
	int ID = getTaskID();


	
	for(Iterator ip=predecessors.iterator(); ip.hasNext();)
	{
		GanttTaskRelationship relation=(GanttTaskRelationship)ip.next();
		
		GanttTask p = relation.getPredecessorTask();
		
		GanttTaskRelationship predRela = new GanttTaskRelationship(
										 -1, ID, relation.getRelationshipType());
		p.removeSuccessor(predRela);		
	}
	
	
	for(Iterator is=successors.iterator(); is.hasNext();)
	{
		GanttTaskRelationship relation=(GanttTaskRelationship)is.next();
		
		GanttTask s = relation.getSuccessorTask();
		
		GanttTaskRelationship succRela = new GanttTaskRelationship(
										 ID, -1, relation.getRelationshipType());
		s.removePredecessor(succRela);
	}
	
	
	successors.clear();
	predecessors.clear();
	
  }
  
  public void removeSuccessor ( GanttTaskRelationship r)
  {
    int index=0;
	for(Iterator is=successors.iterator(); is.hasNext();index++)
	{
		GanttTaskRelationship relation=(GanttTaskRelationship)is.next();
		if( relation.equals(r) ){
			successors.removeElementAt(index);
			return;
		}
	} 
  }
  
  public void removePredecessor ( GanttTaskRelationship r)
  {
     int index=0;
	for(Iterator ip=predecessors.iterator(); ip.hasNext();index++)
	{
		GanttTaskRelationship relation=(GanttTaskRelationship)ip.next();
		if( relation.equals(r) ){
			predecessors.removeElementAt(index);
			return;
		}
	} 
  }
  

  /**return true if the realtionship between this task and others has been checked*/

  public boolean isChecked() {

    return checked;

  }

  /**set the checked state of task: true if the relationship has been check. or else, false*/

  public void setChecked(boolean checked) {

    this.checked = checked;

  }

//End 13-May-2003/ CL
////Added by CL
/////////following code is for taskID management/////////////
  /**ID for the task*/
  private int taskID = -1;
  /**a Hashtable stores all the tasks*/
  //private static Hashtable allTasksMap = new Hashtable();
  /**max ID in the allTasksMap*/
  //private static int maxID = -1;

  public static GanttTask getTaskByTaskID(int taskID) {
    //return (GanttTask) allTasksMap.get(new Integer(taskID));
    return TaskManager.Access.getInstance().getTask(taskID);
  }

  /**
   * @return task ID. -1 if the task has not been registered
   */
  public int getTaskID() {
    return taskID;
  }

  /**
       *set the task ID. the uniquness of ID should be check before using this method
   * @param taskID
   */
  public void setTaskID(int taskID) {
    this.taskID = taskID;
  }
/*
  public static void resetMaxID() {
  	maxID=-1;
	allTasksMap.clear();
  }
*/
  /**
   * be used in loading the data to construct GanttTask object
   * @param taskID ID of the adding task
   * @param newTask adding task
   *//*
  public static void loadTask(int taskID, GanttTask newTask) {
    if (allTasksMap.get(new Integer(taskID)) == null) { //if the taskID is not in the map
      allTasksMap.put(new Integer(taskID), newTask);
      newTask.setTaskID(taskID);
      if (taskID > maxID) {
        maxID = taskID;
      }
    }
    else { //taskID has been in the map. the newTask will not be added
      System.out.println("There is a task that already has the ID " + taskID);
    }
  }*/

  /**
   * the ID for a new task is always one larger than the maximum of
   * the existing ID number. thus guarrantee the uniqueness of ID
   * @param newTask new task for registration
   *//*
  public static void registerATask(GanttTask newTask) {
    ourTaskManager.registerTask(newTask);
  }*/

  /**
   * used when read the information from file
   * @param taskID ID of task
   * @param aTask a task for registration
   *//*
  public static void registerATask(int taskID, GanttTask aTask) {
    //loadTask(taskID, aTask);
    aTask.setTaskID(taskID);
    ourTaskManager.registerTask(aTask);
  }*/
  /**
   * Different with loadTask(int GanttTask), this method force to put a task with ID
   * to the allTasksMap
   * @param taskID ID of the adding task
   * @param newTask adding task
   *//*
  public static void putTaskWithID(int taskID, GanttTask newTask){
    allTasksMap.put(new Integer(taskID), newTask);
    newTask.setTaskID(taskID);
    if (taskID > maxID) {
      maxID = taskID;
    }

  }*/

  /**
   * when open a new file, clear the allTasksMap
   
  public static void clearTasksMap() {
    allTasksMap.clear();
  }*/
  
  private static TaskManager ourTaskManager = TaskManager.Access.getInstance();

}
