/***************************************************************************
                           OptionsPanel.java  -  description
                             -------------------
    begin                : sep 2003
    copyright            : (C) 2003 by Thomas Alexandre
    email                : alexthomas@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



package net.sourceforge.ganttproject.gui;

import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Hashtable;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Container;
import java.awt.Component;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JTabbedPane;
import javax.swing.JDialog;
import javax.swing.JScrollPane;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import java.io.File;
import net.sourceforge.ganttproject.language.GanttLanguage;
import net.sourceforge.ganttproject.roles.RoleManager;
import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.TestGanttRolloverButton;
import javax.swing.filechooser.FileFilter;

/**
  * Dialog to edit options for ganttproject
  */
public class OptionsPanel
	extends JPanel {

	private GanttLanguage language = GanttLanguage.getInstance(); // language the panel will display
	/** The main frame */
	GanttProject appli;
	
	//The dialog
	JDialog dialog;
	
	JTabbedPane tabbedPane;
	
	GridBagConstraints gbc = new GridBagConstraints();
	FlowLayout flowL = new FlowLayout(FlowLayout.LEFT, 10, 10);
	
	//General panel
	JPanel generalPanel;
	JLabel labelAutomatic;
	JCheckBox cbAutomatic;
	JLabel labelRedLine;
	JCheckBox cbRedLine;
	JLabel labelDrag;
	JCheckBox cbDrag;
	JLabel labelXsl;
	JButton buttonXsl;
	JTextField tfXsl;
	JLabel labelXslFo;
	JButton buttonXslFo;
	JTextField tfXslFo;
	JLabel labelColor;
	JButton buttonColor;
	JLabel labelLook;
	JComboBox cbLook;
	JLabel labelLanguage;
	JComboBox cbLanguage;
	//Roles panel
	JPanel settingsPanel;
	JLabel labelName;
	JTextField tfName;
	JLabel labelOrga;
	JTextField tfOrga;
	JLabel labelDescr;
	JTextArea taDescr;
	
	
	//Roles panel
	JPanel rolesPanel;
	JTable rolesTable;

	//South panel
	JButton okButton;
	JButton cancelButton;
	JPanel southPanel;


	/** Constructor */
	public OptionsPanel (JDialog dialog, GanttProject appli) {
		
		this.appli=appli;
		this.dialog=dialog;
		
		init ();
		
		setOptions();
	
	}



	/** Initialisation of the dialog box */
	public void init() {
		tabbedPane = new JTabbedPane();
		
		constructSettingsPanel();
		tabbedPane.add(GanttProject.correctLabel(language.getText("projectProperties")), settingsPanel);
		constructGeneralPanel();
		tabbedPane.add(GanttProject.correctLabel(language.getText("general")), generalPanel);
		constructRolesPanel();
		tabbedPane.add(GanttProject.correctLabel(language.getText("resourceRole")), rolesPanel);
		
		setLayout(new BorderLayout());
		add(tabbedPane, BorderLayout.CENTER);
		constructSouthPanel();
		add(southPanel, BorderLayout.SOUTH);		
	}
	

	/**Construct the general panel*/
	private void constructGeneralPanel() {
		generalPanel = new JPanel(new GridBagLayout());
		labelAutomatic = new JLabel(language.getText("automaticLaunch"));
		labelDrag = new JLabel(language.getText("dragTime"));
		labelRedLine = new JLabel(language.getText("redLine"));
		labelXsl = new JLabel(language.getText("xslDirectory"));
		labelXslFo = new JLabel("XSL-Fo File");
		labelColor = new JLabel(language.getText("defaultTaskColor"));
		labelLook = new JLabel(language.getText("looknfeel"));
		
		cbAutomatic = new JCheckBox();
		cbRedLine = new JCheckBox();
		cbDrag = new JCheckBox();
		buttonXsl = new TestGanttRolloverButton(new ImageIcon(getClass().getResource("/icons/open_16.gif")));
		tfXsl = new JTextField();
		tfXsl.setColumns(20);
		buttonXslFo = new TestGanttRolloverButton(new ImageIcon(getClass().getResource("/icons/open_16.gif")));
		tfXslFo = new JTextField();
		tfXslFo.setColumns(20);
		buttonColor = new JButton(language.getText("colorButton"));
		cbLook = new JComboBox();
		
		// TODO: Maybe this list should be dynamically created by using:
		//       javax.swing.UIManager.getInstalledLookAndFeels()
		//
		//       Before doing this, maybe we have to explicitly add Kunstoff:
		//       javax.swing.UIManager.installLookAndFeel(String name, String className) 
		cbLook.addItem("Metal");
		cbLook.addItem("Mac OS X");
		//cbLook.addItem("Gtk");
		cbLook.addItem("Windows");
		cbLook.addItem("Motif");
		cbLook.addItem("Kunststoff");
		
		labelLanguage = new JLabel(language.getText("language"));
		cbLanguage = new JComboBox();
		cbLanguage.addItem("ChineseBig5");
		cbLanguage.addItem("ChineseGB");
		cbLanguage.addItem("\u010cesky");
		cbLanguage.addItem("Deutsch");
		cbLanguage.addItem("English");
		cbLanguage.addItem("Espa\u00f1ol");
		cbLanguage.addItem("Estonian");
		cbLanguage.addItem("Fran\u00e7ais");
		cbLanguage.addItem("Hungarian");
		cbLanguage.addItem("\u05e2\u05d1\u05e8\u05d9\u05ea");
		cbLanguage.addItem("Italiano");
		cbLanguage.addItem("Japanese");
		cbLanguage.addItem("Nederlands");
		cbLanguage.addItem("Norsk");
		cbLanguage.addItem("Polski");
		cbLanguage.addItem("Portugues");
		cbLanguage.addItem("Portugu\u00eas do Brasil");
		cbLanguage.addItem("\u0420\u0443\u0441\u0441\u043a\u0438\u0439");
		cbLanguage.addItem("Svenska");
		cbLanguage.addItem("T\u00FCrk\u00E7e");
		
		
		
		JPanel line1 = new JPanel(flowL);
		JPanel line2 = new JPanel(flowL);
		JPanel line3 = new JPanel(flowL);
		JPanel line4 = new JPanel(flowL);
		JPanel line5 = new JPanel(flowL);
		JPanel line6 = new JPanel(flowL);
		
		line1.setLayout(new GridBagLayout());
		gbc.weightx = 0;
    	gbc.weighty = 0;
    	gbc.anchor = GridBagConstraints.WEST;
	    gbc.insets.right = 15;
	    gbc.insets.left = 10;
	    gbc.insets.top = 10;
		addUsingGBL(line1, labelAutomatic, gbc, 0, 0, 2, 1);
		addUsingGBL(line1, cbAutomatic, gbc, 2, 0, 1, 1);		
		addUsingGBL(line1, labelDrag, gbc, 3, 0, 2, 1);
		addUsingGBL(line1, cbDrag, gbc, 5, 0, 1, 1);
		
		line2.setLayout(new GridBagLayout());
		addUsingGBL(line2, labelRedLine, gbc, 0, 0, 2, 1);
		addUsingGBL(line2, cbRedLine, gbc, 2, 0, 1, 1);		
		
		line3.setLayout(new GridBagLayout());
		addUsingGBL(line3, labelXsl, gbc, 0, 0, 1, 1);
		addUsingGBL(line3, tfXsl, gbc, 1, 0, 1, 1);
		addUsingGBL(line3, buttonXsl, gbc, 2, 0, 1, 1);
		line3.setBorder(new TitledBorder(new EtchedBorder(), language.getText("htmlexport")));
		
		line4.setLayout(new GridBagLayout());
		addUsingGBL(line4, labelXslFo, gbc, 0, 0, 1, 1);
		addUsingGBL(line4, tfXslFo, gbc, 1, 0, 1, 1);
		addUsingGBL(line4, buttonXslFo, gbc, 2, 0, 1, 1);
		line4.setBorder(new TitledBorder(new EtchedBorder(), language.getText("pdfexport")));
		
		
		line5.setLayout(new GridBagLayout());
		addUsingGBL(line5, labelColor, gbc, 0, 0, 2, 1);
		addUsingGBL(line5, buttonColor, gbc, 2, 0, 1, 1);
		
		line6.setLayout(new GridBagLayout());
		addUsingGBL(line6, labelLook, gbc, 0, 0, 1, 1);
		addUsingGBL(line6, cbLook, gbc, 1, 0, 1, 1);
		addUsingGBL(line6, labelLanguage, gbc, 2, 0, 1, 1);
		addUsingGBL(line6, cbLanguage, gbc, 3, 0, 1, 1);
	
	
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets.right = 15;
		gbc.insets.left = 10;
		gbc.insets.top = 10;
		addUsingGBL(generalPanel, line1, gbc, 0, 0, 1, 1);
		addUsingGBL(generalPanel, line2, gbc, 0, 1, 1, 1);
		addUsingGBL(generalPanel, line3, gbc, 0, 2, 1, 1);
		addUsingGBL(generalPanel, line4, gbc, 0, 3, 1, 1);
		addUsingGBL(generalPanel, line5, gbc, 0, 4, 1, 1);
		addUsingGBL(generalPanel, line6, gbc, 0, 5, 1, 1);
		
		
		final String colorChooserTitle = language.getText("selectColor");
		buttonColor.addActionListener(new ActionListener() {
      		public void actionPerformed(ActionEvent e) {
		        JDialog dialog;
		        dialog = JColorChooser.createDialog(appli, colorChooserTitle, true,
		                                            GanttDialogOptions.colorChooser,new ActionListener() {
			          public void actionPerformed(ActionEvent e) {
				            buttonColor.setBackground(GanttDialogOptions.colorChooser.getColor());
		    	      }
		        }
        	, new ActionListener() {
		          public void actionPerformed(ActionEvent e) {
		            // nothing to do for "Cancel"
		          }
        	});
        	dialog.show();
      	  }
    	});
	
	
		buttonXsl.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser(new File(tfXsl.getText()));
				fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int returnVal = fc.showSaveDialog(dialog);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					tfXsl.setText(fc.getSelectedFile().toString());
				}
			}
		});
		
		buttonXslFo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				FileFilter xslFilter = new XSLFileFilter();
				JFileChooser fc = new JFileChooser(new File(tfXslFo.getText()));
				fc.addChoosableFileFilter(xslFilter);
				int returnVal = fc.showSaveDialog(dialog);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					tfXslFo.setText(fc.getSelectedFile().toString());
				}
			}
		});
	
	
	}
	
	/**Construct the roles panel*/
	private void constructRolesPanel() {
		rolesPanel = new JPanel(new GridBagLayout());
		RolesTableModel rolesModel = new RolesTableModel();
		rolesTable = new JTable(rolesModel);
		rolesTable.setPreferredScrollableViewportSize(new Dimension(500, 150));
		rolesTable.setRowHeight(23);
		rolesTable.getColumnModel().getColumn(0).setPreferredWidth(30);
		rolesTable.getColumnModel().getColumn(1).setPreferredWidth(470);
		rolesModel.addTableModelListener( new TableModelListener(){
			public void tableChanged(TableModelEvent e) {
				//System.out.println("Editing a new cell "+rolesTable.getValueAt(e.getFirstRow(),1) );
			}
		});
		
		
		
		/*String [] roles = RoleManager.Access.getInstance().getRoles();
		for(int i=0;i<roles.length;i++)
			System.out.println(roles[i]);*/
		
		
		JScrollPane rolesScrollPane = new JScrollPane(rolesTable);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets.right = 15;
		gbc.insets.left = 10;
		gbc.insets.top = 10;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 8;
		gbc.gridheight = 1;
		gbc.weighty = 1;
		rolesPanel.add(rolesScrollPane, gbc);	
	}
	
	/**Construct the general panel*/
	private void constructSettingsPanel() {
		settingsPanel = new JPanel(new GridBagLayout());
		labelName = new JLabel(language.getText("name"));
		tfName = new JTextField();
		tfName.setColumns(20);
		labelOrga = new JLabel(language.getText("organization"));
		tfOrga = new JTextField();
		tfOrga.setColumns(20);
		labelDescr = new JLabel(language.getText("shortDescription"));
		taDescr = new JTextArea(10,20);
		taDescr.setLineWrap(true);
   		taDescr.setWrapStyleWord(true);
		JScrollPane scpArea0 = new JScrollPane(taDescr);
   
		JPanel panel = new JPanel(new GridBagLayout());
		gbc.weightx = 0;
    	gbc.weighty = 0;
    	gbc.anchor = GridBagConstraints.WEST;
	    gbc.insets.right = 15;
	    gbc.insets.left = 10;
	    gbc.insets.top = 10;
		
		
		addUsingGBL(panel, labelName, gbc, 0, 0, 1, 1);
		gbc.anchor = GridBagConstraints.NORTH;
		addUsingGBL(panel, tfName, gbc, 0, 1, 1, 1);
		//addUsingGBL(new JLabel(""), tfName, gbc, 0, 2, 1, 1);
		//addUsingGBL(new JLabel(""), tfName, gbc, 0, 3, 1, 1);
		gbc.anchor = GridBagConstraints.WEST;
		addUsingGBL(panel, labelOrga, gbc, 0, 4, 1, 1);
		gbc.anchor = GridBagConstraints.NORTH;
		addUsingGBL(panel, tfOrga, gbc, 0, 5, 1, 1);
		gbc.anchor = GridBagConstraints.WEST;
		addUsingGBL(panel, labelDescr, gbc, 1, 0, 1, 1);
		gbc.anchor = GridBagConstraints.NORTH;
		addUsingGBL(panel, scpArea0, gbc, 1, 1, 1, 5);
		
		
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets.right = 15;
		gbc.insets.left = 10;
		gbc.insets.top = 10;
		addUsingGBL(settingsPanel, panel, gbc, 0, 0, 1, 1);
	}
	
	
	/** Construct panel with ok and cancel button*/
	private void constructSouthPanel() {
		okButton = new JButton(language.getText("ok")/*, new ImageIcon(getClass().getResource("/icons/yes.gif"))*/);
		okButton.setName("ok");
		cancelButton = new JButton(language.getText("cancel")/*, new ImageIcon(getClass().getResource("/icons/no.gif"))*/);
		cancelButton.setName("cancel");
		southPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 50, 10));
		southPanel.add(okButton);
		southPanel.add(cancelButton);
	}


	/**add a component to container by using GridBagConstraints.*/
	private void addUsingGBL(Container container, Component component,
							GridBagConstraints gbc, int x,
							int y, int w, int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		gbc.weighty = 0;
		container.add(component, gbc);
	}


	/** Add the differents action listener on the differents widgets */
	public void addActionListener(ActionListener l) {
		okButton.addActionListener(l);
		cancelButton.addActionListener(l);
	}
	
	/** Set the option value to the dialog */
	private void setOptions() {
		tfName.setText(appli.projectName);
		tfOrga.setText(appli.organization);
		taDescr.setText(appli.description);
		
			
		buttonColor.setBackground(new Color(appli.getArea().getTaskColor().getRed(),
                            appli.getArea().getTaskColor().getGreen(),
                            appli.getArea().getTaskColor().getBlue()));
		cbLook.setSelectedItem(appli.styleName);
		cbAutomatic.setSelected(appli.automatic);
		cbRedLine.setSelected(appli.getArea().redline);
		cbDrag.setSelected(appli.dragTime);
		cbLanguage.setSelectedItem(language.getText("longLanguage"));
		try {
			tfXsl.setText(GanttProject.xslDir!=null?GanttProject.xslDir:System.getProperty("user.home"));
		} catch (AccessControlException e) {
			// This can happen when running in a sandbox (Java WebStart)
			System.err.println (e + ": " + e.getMessage());
		}	
		try {
			tfXslFo.setText(GanttProject.xslFo!=null?GanttProject.xslFo:System.getProperty("user.home"));
		} catch (AccessControlException e) {
			// This can happen when running in a sandbox (Java WebStart)
			System.err.println (e + ": " + e.getMessage());
		}									
	}

	/** Return the selected project name */
	public String getProjectName() {
		return tfName.getText();
	}

	/** Return the ogranisation */
	public String getProjectOrganization() {
		return tfOrga.getText();
	}

	/** Return the project description */
	public String getProjectDescription() {
		return taDescr.getText();
	}
	
	/** Return the default task color */
	public Color getDefaultColor() {
		return buttonColor.getBackground();
	}
	
	public boolean getAutomatic() {
		return cbAutomatic.isSelected();
	}
	
	public boolean getDragTime() {
		return cbDrag.isSelected();
	}
	
	public boolean getRedLine() {
		return cbRedLine.isSelected();
	}
	
	
	/** Return the xsl directory */
	public String getXslDirectory() {
		if(tfXsl.getText().length()>0 && new File(tfXsl.getText()).exists())
			return tfXsl.getText();
		
		return OptionsPanel.class.getResource("/xslt").toString();
	}
	
	/** Return the xslfo file */
	public String getXslFoFile() {
		if(tfXslFo.getText().length()>0 && new File(tfXslFo.getText()).exists())
			return tfXslFo.getText();
		
		return OptionsPanel.class.getResource("/xslfo/ganttproject.xsl").toString();
	}
	
	/** Return the class of the style */
	public String getStyleClass() {
		String styleName = getStyleName();
		if(styleName.equals("Metal")) return "javax.swing.plaf.metal.MetalLookAndFeel";
		if(styleName.equals("Mac OS X")) return "apple.laf.AquaLookAndFeel";
		//if(styleName.equals("Gtk")) return "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
		if(styleName.equals("Windows")) return "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
		if(styleName.equals("Motif")) return "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
		/*if(styleName.equals("Kunststoff"))*/ return "com.incors.plaf.kunststoff.KunststoffLookAndFeel";
	}
	
	/** Return the name of the style */
	public String getStyleName() {
		return (String)cbLook.getSelectedItem();
	}
	
	/**Appli the new language*/
	public void changeLanguage(){
		String lang=(String)cbLanguage.getSelectedItem();
		
		
		if(lang.equals("ChineseBig5"))  language.setLocale(new Locale("zhb5", "CN"));
		else if(lang.equals("ChineseGB"))  language.setLocale(new Locale("zhgb", "CN"));
		else if(lang.equals("\u010cesky"))  language.setLocale(new Locale("cz", "CZ"));
		else if(lang.equals("Deutsch"))  language.setLocale(Locale.GERMANY);
		else if(lang.equals("English"))  language.setLocale(Locale.US);
		else if(lang.equals("Espa\u00f1ol"))  language.setLocale(new Locale("es", "ES"));
		else if(lang.equals("Estonian"))  language.setLocale(new Locale("et", "ET"));
		else if(lang.equals("Fran\u00e7ais"))  language.setLocale(Locale.FRANCE);
		else if(lang.equals("Hungarian"))  language.setLocale(new Locale("hu", "HU"));
		else if(lang.equals("\u05e2\u05d1\u05e8\u05d9\u05ea"))  language.setLocale(new Locale("iw", "iW"));
		else if(lang.equals("Italiano"))  language.setLocale(Locale.ITALY);
		else if(lang.equals("Japanese"))  language.setLocale(new Locale("ja", "JP"));
		else if(lang.equals("Nederlands"))  language.setLocale(new Locale("nl", "NL"));
		else if(lang.equals("Norsk"))  language.setLocale(new Locale("no", "NO"));
		else if(lang.equals("Polski"))  language.setLocale(new Locale("pl", "PL"));
		else if(lang.equals("Portugues"))  language.setLocale(new Locale("pt", "PT"));
		else if(lang.equals("Portugu\u00eas do Brasil"))  language.setLocale(new Locale("pt", "BR"));
		else if(lang.equals("Svenska"))  language.setLocale(new Locale("sv", "SV"));
		else if(lang.equals("T\u00FCrk\u00E7e"))  language.setLocale(new Locale("tr", "TR"));
		else if(lang.equals("\u0420\u0443\u0441\u0441\u043a\u0438\u0439"))  language.setLocale(new Locale("ru", "RU"));
		appli.changeLanguage();
	}
	
	/** Change the role on the role manager */
	public void setRoles(){
		TableModel model = rolesTable.getModel();
	    
		Hashtable mapRoles = new Hashtable();
		
		int id=0;
		for (int i = 11; i < model.getRowCount(); i++) {
			if(model.getValueAt(i, 1)!=null){
				String roleName = model.getValueAt(i, 1).toString(); 
				if(roleName.length()!=0)
					mapRoles.put(new Integer(id++), roleName);
			}
		}

		RoleManager.Access.getInstance().setRoles(mapRoles);
	}
	
  
	/** The filter for xsl file */
  public class XSLFileFilter
      extends FileFilter {
    public boolean accept(File f) {
      return f.getName().toLowerCase().endsWith(".xsl") || f.isDirectory();
    }

    public String getDescription() {
      return language.getText("filterxsl");
    }
  }
	
}



/**
 * Model of table for roles
 */
class RolesTableModel
    extends AbstractTableModel {

  final String[] columnNames = {
      GanttLanguage.getInstance().getText("id"),
      GanttLanguage.getInstance().getText("resourceRole")};
  final Object[][] data = new Object[100][];

  /**Constructor */
  public RolesTableModel() {

    String []listOfRoles = RoleManager.Access.getInstance().getRoles();//GanttLanguage.getInstance().getPersonFunction();
	int i;
	for(i=0;i<listOfRoles.length;i++)
		data[i] = new Object[] {
			String.valueOf(i),
			listOfRoles[i]};
	for (int j = i; j < data.length; j++) {
      data[j] = new Object[2]; // {"", ""};
    }
  }

  /** Return the number of colums */
  public int getColumnCount() {
    return columnNames.length;
  }

  /** Return the number of rows */
  public int getRowCount() {
    return data.length;
  }

  /** Return the name of the column at col index */
  public String getColumnName(int col) {
    return columnNames[col];
  }

  /** Return the object a specify cell */
  public Object getValueAt(int row, int col) {
    return data[row][col];

  }

  /*
   * JTable uses this method to determine the default renderer/
   * editor for each cell.  If we didn't implement this method,
   * then the last column would contain text ("true"/"false"),
   * rather than a check box.
   */

  /*public Class getColumnClass(int c) {
    if (c == 0 || c == 2) {
      return new String().getClass();
    }
    else {
      return new HumanResource().getClass();
    }
//        return getValueAt(0, c).getClass();
  }*/

  public boolean isCellEditable(int row, int col) {
    return col!=0 && row>10;
  }

  /*
   * Don't need to implement this method unless your table's
   * data can change.
   */
  public void setValueAt(Object value, int row, int col) {
    data[row][col] = value;
    fireTableCellUpdated(row, col);
  }

}









